clc;
clear;
close all;
%
% We declare as global the time series for the trend components of consumption, 
% output and hours worked, as obtained from the Hodrick and Prescott filter
global lyt lct lht
%
% Exogenous parameters are dcelared as global variables
global param contT
%
% We load the data and apply the HP filter 
load ych.dat -ascii
lct=log(ych(:,2));
lyt=log(ych(:,1));
lht=log(ych(:,3));
%
contT=0; %if contT==0, the vector of observable variables is consumption and hours. It woill be output and hours otherwise.
%
% We eliminate trend components and take differences to the mean   
    trend=1:length(lct);
    clyt=lyt-0.0055457*trend'; 
    lyt=clyt-mean(clyt);
    clct=lct-0.004995*trend'; 
    lct=clct-mean(clct);
    lht=lht-mean(lht);
T=length(lyt);  % sample size
%%%%%%%%%%
%
% We load the variance-covariance matrix
load mcov.prn -ascii
ind_mcov=0;   % if it is equal to zero, we do not use this matrix
%
% Model parameters
thetab = 1.6;    % Average productivity shock 
pbeta = 0.99;    % discount rate
delta = 0.025;   % capital depreciation rate  
alpha = 0.26;    % output elasticity with respect to capital 
rho = 0.95;      % persistency of the productivity shock
sigmae = 0.02;   % standard deviation of the innovation to the productivity shock
rhoa = 0.95;     % Persistence of shock in preferences
sigmaa = 0.02;   % standard deviation of innovation to the shock in preferences 
pgamma = 0.0045; % slope of Firsch labour supply
param = [pbeta delta];
Theta = [thetab alpha rho sigmae rhoa sigmaa pgamma];
%fig=cellstr(strvcat('thetab','alpha','rho','sigmae','rhoa','sigmaa','pgamma'));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Number of sample realizations
N = 30000;            % number of sample paths for the posterior distribution
Nprior = 30000;       % number of sample paths for the prior distribution
dN = 0.3*N;           % number of paths that will be deleted to avoid a possible initial effect bias
%%%%%%%%%%%%%%%%%%%%%
%
% Quantiles to be computed
vquant = [0.025  0.975];
%%%%%%%%%%%%%%%%%%%%%
%
% Covariance matrix to compute the parameter vector to be used in the posterior distribution
if ind_mcov==0
    cc=0.004;
    Msigma=cc.*eye(length(Theta)); % it can be changed by the variance-covariance matrix estimated by a frequentist approach
else
    cc=0.6;
    Msigma=cc.*mcov;
end
Pchol=chol(Msigma);   % Notice that  Pchol'*Pchol=Msigma
%
% Prior distributions of the variables to be estimated
% We specify the hyperparameters of the prior distributions
    %hyp = ones(length(paramest),2);
    hyp_theta = [2.69^2  1/2.69]; % Gamma(a,b) distribution for thetab; a=hyp(:,1), b=hyp(:,2); E(thetab)=a*b; V(thetab)=a*b^2
    hyp_alpha = [120 240];   %24 por 36       % Beta(a,b) distribution for alpha; a=hyp(:,1), b=hyp(:,2); E(alpha)=a/(a+b); V(thetab)=a*b/[(a+b+1)*(a+b)^2]
    hyp_rho   = [900 100]; %[7 3];          % Beta(a,b) distribution for rho; a=hyp(:,1), b=hyp(:,2); E(rho)=a/(a+b); V(rho)=a*b/[(a+b+1)*(a+b)^2]
    hyp_sigmae= [32 1/1600];      % Gamma(a,b) distribution for sigmae; a=hyp(:,1), b=hyp(:,2); E(sigmae)=a*b; V(sigmae)=a*b^2
    hyp_rhoa  = [900 100];          % Beta(a,b) distribution for rhoa; a=hyp(:,1), b=hyp(:,2); E(rhoa)=a/(a+b); V(rhoa)=a*b/[(a+b+1)*(a+b)^2]
    hyp_sigmaa= [15 1/1000];      % Gamma(a,b) distribution for sigmaa; a=hyp(:,1), b=hyp(:,2); E(sigmaa)=a*b; V(sigmaa)=a*b^2
    hyp_gamma = [1 0.004];%[1 0.0028];     % Gamma(a,b) distribution for pgamma; a=hyp(:,1), b=hyp(:,2); E(pgamma)=a*b; V(pgamma)=a*b^2
%
% Random number generation of the prior distributions, to be graphed together with the 
% posterior distribution
ptheta_prior=gamrnd(hyp_theta(1),hyp_theta(2),Nprior,1); 
palpha_prior=betarnd(hyp_alpha(1),hyp_alpha(2),Nprior,1);
prho_prior=betarnd(hyp_rho(1),hyp_rho(2),Nprior,1);
psigmae_prior=gamrnd(hyp_sigmae(1),hyp_sigmae(2),Nprior,1);
prhoa_prior=betarnd(hyp_rhoa(1),hyp_rhoa(2),Nprior,1);
psigmaa_prior=gamrnd(hyp_sigmaa(1),hyp_sigmaa(2),Nprior,1);
pgamma_prior=gamrnd(hyp_gamma(1),hyp_gamma(2),Nprior,1);
%
paramprior=[ptheta_prior palpha_prior prho_prior psigmae_prior...
            prhoa_prior psigmaa_prior pgamma_prior];   % We keep these sample realizations to draw histograms later on 
Theta0 =[mean(ptheta_prior) mean(palpha_prior) mean(prho_prior) ...
         mean(psigmae_prior) mean(prhoa_prior) mean(psigmaa_prior) ...
         mean(pgamma_prior)]';
% Notice that we estimate and draw sample realizations of the transformed 
% parameters, to make sure that they belong to the feasible parameter space
Thetatr0 =[log(Theta0(1));
           log(Theta0(2)/(1-Theta0(2)));
           log(Theta0(3)/(1-Theta0(3)));
           log(Theta0(4));
           log(Theta0(5)/(1-Theta0(5)));
           log(Theta0(6));
           log(Theta0(7))];  % transformed parameters
%
% We compute the log of the joint density for the 7 parameters
lnprior0=(hyp_theta(1)-1)*log(Theta0(1))-Theta0(1)/hyp_theta(2)+...
         (hyp_alpha(1)-1)*log(Theta0(2))+(hyp_alpha(2)-1)*log(1-Theta0(2))+...
         (hyp_rho(1)-1)*log(Theta0(3))+(hyp_rho(2)-1)*log(1-Theta0(3))+...
         (hyp_sigmae(1)-1)*log(Theta0(4))-Theta0(4)/hyp_sigmae(2)+...
         (hyp_rhoa(1)-1)*log(Theta0(5))+(hyp_rhoa(2)-1)*log(1-Theta0(5))+...
         (hyp_sigmaa(1)-1)*log(Theta0(6))-Theta0(6)/hyp_sigmaa(2)+...
         (hyp_gamma(1)-1)*log(Theta0(7))-Theta0(7)/hyp_gamma(2);
%
% We compute the derivatives of the logged transformed parameters 
lnjacob=log(Theta0(1))+log(Theta0(2))+log(1-Theta0(2))+...
        log(Theta0(3))+log(1-Theta0(3))+log(Theta0(4))+...
        log(Theta0(5))+log(1-Theta0(5))+log(Theta0(6))+...
        log(Theta0(7));
%
% We compute the prior distribution of the transformed variable
lnprior0=lnprior0+lnjacob;
% We compute the likelihood of the transformed parameters
[A,B,C,D]=hansen_model(Theta0);
lfv0=-lfvkf(A,B,C,D);
%
Prob_Acept=0;   % We initialize the probability of acceptance at zero
postdraws=zeros(N+dN,2*length(Theta0)+5); % We save the parameters of the posterior distribution, the acceptance probability, the prior densities and their likelihood
postdraws_dv_C=zeros(N+dN,14);
postdraws_dv_Y=zeros(N+dN,14);
postdraws_dv_H=zeros(N+dN,14);
postdraws_dv_I=zeros(N+dN,14);
postdraws_dv_W=zeros(N+dN,14);
postdraws_dv_i=zeros(N+dN,14);
postdraws_imps=zeros(N+dN,800);
postdraws_impd=zeros(N+dN,800);
N_Prob_Acept=0; % We accumulate the number of acceptances, starting at zero
timer=clock;   % We start the clock to compute how long it takes to compute a posterior distribution
% Metropolis-Hastings Algorithm
for i=1:N+dN
    % take a candidate draw
    eps=randn(length(Theta0),1);
    eps1=Pchol'*eps;
    Thetatr1=Thetatr0+eps1;
    Theta1=[exp(Thetatr1(1));
            exp(Thetatr1(2))/(1+exp(Thetatr1(2)));
            exp(Thetatr1(3))/(1+exp(Thetatr1(3)));
            exp(Thetatr1(4));
            exp(Thetatr1(5))/(1+exp(Thetatr1(5)));
            exp(Thetatr1(6));
            exp(Thetatr1(7))];
    lnprior1=(hyp_theta(1)-1)*log(Theta1(1))-Theta1(1)/hyp_theta(2)+...
         (hyp_alpha(1)-1)*log(Theta1(2))+(hyp_alpha(2)-1)*log(1-Theta1(2))+...
         (hyp_rho(1)-1)*log(Theta1(3))+(hyp_rho(2)-1)*log(1-Theta1(3))+...
         (hyp_sigmae(1)-1)*log(Theta1(4))-Theta1(4)/hyp_sigmae(2)+...
         (hyp_rhoa(1)-1)*log(Theta1(5))+(hyp_rhoa(2)-1)*log(1-Theta1(5))+...
         (hyp_sigmaa(1)-1)*log(Theta1(6))-Theta1(6)/hyp_sigmaa(2)+...
         (hyp_gamma(1)-1)*log(Theta1(7))-Theta1(7)/hyp_gamma(2);
    %
    lnjacob=log(Theta1(1))+log(Theta1(2))+log(1-Theta1(2))+...
            log(Theta1(3))+log(1-Theta1(3))+log(Theta1(4))+...
            log(Theta1(5))+log(1-Theta1(5))+log(Theta1(6))+...
            log(Theta1(7));
    lnprior1=lnprior1+lnjacob;
    [A,B,C,D]=hansen_model(Theta1);
    lfv1=-lfvkf(A,B,C,D);
    Prob_Acept=exp(lnprior1+lfv1-lnprior0-lfv0);
    U=rand(1,1)<Prob_Acept;
    N_Prob_Acept=N_Prob_Acept+U;
    Theta0=U*Theta1+(1-U)*Theta0;
    Thetatr0=U*Thetatr1+(1-U)*Thetatr0;
    lnprior0=U*lnprior1+(1-U)*lnprior0;
    lfv0=U*lfv1+(1-U)*lfv0;
    postdraws(i,:)=[Theta0' Thetatr0' Prob_Acept lnprior0 lnprior1 lfv0 lfv1];
    % Variance decomposition
    [Ax,Bx,Cx,V]=state_space_matrices(Theta0);
    [V_variance, Mdecomp_v]=decomp_var(Ax,Bx,Cx,V);
    supply_shock_C=Mdecomp_v(1:7,1);
    demand_shock_C=Mdecomp_v(1:7,2);
    postdraws_dv_C(i,:)=[supply_shock_C' demand_shock_C'];
    
    supply_shock_Y=Mdecomp_v(8:14,1);
    demand_shock_Y=Mdecomp_v(8:14,2);
    postdraws_dv_Y(i,:)=[supply_shock_Y' demand_shock_Y'];
    
    supply_shock_H=Mdecomp_v(15:21,1);
    demand_shock_H=Mdecomp_v(15:21,2);
    postdraws_dv_H(i,:)=[supply_shock_H' demand_shock_H'];
    
    supply_shock_I=Mdecomp_v(22:28,1);
    demand_shock_I=Mdecomp_v(22:28,2);
    postdraws_dv_I(i,:)=[supply_shock_I' demand_shock_I'];
    
    supply_shock_W=Mdecomp_v(29:35,1);
    demand_shock_W=Mdecomp_v(29:35,2);
    postdraws_dv_W(i,:)=[supply_shock_W' demand_shock_W'];
    
    supply_shock_i=Mdecomp_v(36:42,1);
    demand_shock_i=Mdecomp_v(36:42,2);
    postdraws_dv_i(i,:)=[supply_shock_i' demand_shock_i'];
    %
    % Impulse responses
    % supply shock
        innovs=zeros(99,2);
        innovs(10,1)=Theta0(4);
        imps=irf(Theta0,innovs);
        imps=imps*100;
        postdraws_imps(i,:)=imps(:)';
    % demand shock
        innovd=zeros(99,2);
        innovd(10,2)=Theta0(6);
        impd=irf(Theta0,innovd);
        impd=impd*100;
        postdraws_impd(i,:)=impd(:)';
    
    [i/(N+dN)*100 N_Prob_Acept/i]
end
disp('Time elapsed in generating the posterior');
timer=etime(clock,timer)
disp('Aceptance probability');
N_Prob_Acept=N_Prob_Acept/(N+dN)
postdraws=postdraws(dN+1:N+dN,:);
postdraws_dv_C=postdraws_dv_C(dN+1:N+dN,:);
postdraws_dv_Y=postdraws_dv_Y(dN+1:N+dN,:);
postdraws_dv_H=postdraws_dv_H(dN+1:N+dN,:);
postdraws_dv_I=postdraws_dv_I(dN+1:N+dN,:);
postdraws_dv_W=postdraws_dv_W(dN+1:N+dN,:);
postdraws_dv_i=postdraws_dv_i(dN+1:N+dN,:);
postdraws_imps=postdraws_imps(dN+1:N+dN,:);
postdraws_impd=postdraws_impd(dN+1:N+dN,:);
V=cov(postdraws(:,length(Theta0)+1:2*length(Theta0)));
figure;
for i=1:length(Theta0)
    subplot(3,3,i);
    h1=histogram(paramprior(:,i));
    h1.FaceColor='y';
    h1.EdgeColor='none';
    h1.Normalization='pdf';
    hold on
    h2=histogram(postdraws(:,i));
    h2.FaceColor='b';
    h2.EdgeColor='none';
    h2.Normalization='pdf';
    vv=axis;
    line(mean(postdraws(:,i))*ones(1,2),[vv(3) vv(4)],'Linewidth',2,'Color','b');
    line(mean(paramprior(:,i))*ones(1,2),[vv(3) vv(4)],'Linewidth',2,'Color','y');
    hold off
    if i==1
        title('\fontsize{14} \theta');
    elseif i==2
        title('\fontsize{14} \alpha');
    elseif i==3
        title('\fontsize{14} \rho');
    elseif i==4
        title('\fontsize{14} \sigma');
    elseif i==5
        title('\fontsize{14} \rho_{\ita}');
    elseif i==6
        title('\fontsize{14} \sigma_\zeta');
    elseif i==7
        title('\fontsize{14} \gamma');
    end
    if i==length(Theta0)
        legend('posterior','prior','mean-posterior','mean-prior');
    end
end
%
if ind_mcov==0
    save mcov.prn V -ascii
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
format short
names_var={' bar_theta ';' alpha ';' rho ';' sigma ';' rho_a ';' sigma_z ';' gamma '};
mean_posterior=mean(postdraws(:,1:7))';
perc025=prctile(postdraws(:,1:7),vquant(1)*100);   
perc975=prctile(postdraws(:,1:7),vquant(2)*100);
percentile02_5=perc025';
percentile97_5=perc975';
disp('====================================================================')
disp('Table 1. Bayesian Inference')
Tabla=table(mean_posterior,percentile02_5,percentile97_5,...
            'RowNames',names_var)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Tables: Variance decompositions
quarters=[1;4;8;12;20;40;inf];
names_var2={'1-step forecast';'4-step forecast';'8-step forecast';
    '12-step forecast';'20-step forecast';'40-step forecast';
    'inconditional variance (inf-step forecast)'};
%
supply_shock=mean(postdraws_dv_C(:,1:7))';
perc02_5_s=prctile(postdraws_dv_C(:,1:7),vquant(1)*100)';
perc97_5_s=prctile(postdraws_dv_C(:,1:7),vquant(2)*100)';
demand_shock=mean(postdraws_dv_C(:,8:14))';
perc02_5_d=prctile(postdraws_dv_C(:,8:14),vquant(1)*100)';
perc97_5_d=prctile(postdraws_dv_C(:,8:14),vquant(2)*100)';
disp('====================================================================')
disp('Table 2.Forecast error variance decompositions. Consumption')
TABLA=table(quarters,supply_shock,perc02_5_s,perc97_5_s,demand_shock,perc02_5_d,perc97_5_d,'RowNames',names_var2)
%
supply_shock=mean(postdraws_dv_Y(:,1:7))';
perc02_5_s=prctile(postdraws_dv_Y(:,1:7),vquant(1)*100)';
perc97_5_s=prctile(postdraws_dv_Y(:,1:7),vquant(2)*100)';
demand_shock=mean(postdraws_dv_Y(:,8:14))';
perc02_5_d=prctile(postdraws_dv_Y(:,8:14),vquant(1)*100)';
perc97_5_d=prctile(postdraws_dv_Y(:,8:14),vquant(2)*100)';
disp('====================================================================')
disp('Table 3.Forecast error variance decompositions. Output')
TABLA=table(quarters,supply_shock,perc02_5_s,perc97_5_s,demand_shock,perc02_5_d,perc97_5_d,'RowNames',names_var2)
%
supply_shock=mean(postdraws_dv_H(:,1:7))';
perc02_5_s=prctile(postdraws_dv_H(:,1:7),vquant(1)*100)';
perc97_5_s=prctile(postdraws_dv_H(:,1:7),vquant(2)*100)';
demand_shock=mean(postdraws_dv_H(:,8:14))';
perc02_5_d=prctile(postdraws_dv_H(:,8:14),vquant(1)*100)';
perc97_5_d=prctile(postdraws_dv_H(:,8:14),vquant(2)*100)';
disp('====================================================================')
disp('Table 4.Forecast error variance decompositions. Hours')
TABLA=table(quarters,supply_shock,perc02_5_s,perc97_5_s,demand_shock,perc02_5_d,perc97_5_d,'RowNames',names_var2)
%
supply_shock=mean(postdraws_dv_I(:,1:7))';
perc02_5_s=prctile(postdraws_dv_I(:,1:7),vquant(1)*100)';
perc97_5_s=prctile(postdraws_dv_I(:,1:7),vquant(2)*100)';
demand_shock=mean(postdraws_dv_I(:,8:14))';
perc02_5_d=prctile(postdraws_dv_I(:,8:14),vquant(1)*100)';
perc97_5_d=prctile(postdraws_dv_I(:,8:14),vquant(2)*100)';
disp('====================================================================')
disp('Table 5.Forecast error variance decompositions. Investment')
TABLA=table(quarters,supply_shock,perc02_5_s,perc97_5_s,demand_shock,perc02_5_d,perc97_5_d,'RowNames',names_var2)
%
supply_shock=mean(postdraws_dv_W(:,1:7))';
perc02_5_s=prctile(postdraws_dv_W(:,1:7),vquant(1)*100)';
perc97_5_s=prctile(postdraws_dv_W(:,1:7),vquant(2)*100)';
demand_shock=mean(postdraws_dv_W(:,8:14))';
perc02_5_d=prctile(postdraws_dv_W(:,8:14),vquant(1)*100)';
perc97_5_d=prctile(postdraws_dv_W(:,8:14),vquant(2)*100)';
disp('====================================================================')
disp('Table 6.Forecast error variance decompositions. Wages')
TABLA=table(quarters,supply_shock,perc02_5_s,perc97_5_s,demand_shock,perc02_5_d,perc97_5_d,'RowNames',names_var2)
%
supply_shock=mean(postdraws_dv_i(:,1:7))';
perc02_5_s=prctile(postdraws_dv_i(:,1:7),vquant(1)*100)';
perc97_5_s=prctile(postdraws_dv_i(:,1:7),vquant(2)*100)';
demand_shock=mean(postdraws_dv_i(:,8:14))';
perc02_5_d=prctile(postdraws_dv_i(:,8:14),vquant(1)*100)';
perc97_5_d=prctile(postdraws_dv_i(:,8:14),vquant(2)*100)';
disp('====================================================================')
disp('Table 7.Forecast error variance decompositions. Interest Rate')
TABLA=table(quarters,supply_shock,perc02_5_s,perc97_5_s,demand_shock,perc02_5_d,perc97_5_d,'RowNames',names_var2)
%
%Impulse responses. Figures
perc025=prctile(postdraws_imps,vquant(1)*100)';perc025=reshape(perc025,100,8);
perc975=prctile(postdraws_imps,vquant(2)*100)';perc975=reshape(perc975,100,8);
mean_imp=mean(postdraws_imps)';mean_imp=reshape(mean_imp,100,8);
%Graphs
figure;
for i=1:8
    hold on
    subplot(4,2,i)
    H1=plot([mean_imp(:,i) perc975(:,i) perc025(:,i)]);
    xlabel('periods');
    set(H1(1),'Color','k','LineStyle','-');
    set(H1(2),'Color','r','LineStyle','--');
    set(H1(3),'Color','r','LineStyle','--');
    if i==1
        title('capital (%)');
    elseif i==2
        title('productivity shock (%)');
    elseif i==3
        title('preferences shock (%)');
    elseif i==4
        title('consumption (%)');
    elseif i==5
        title('output (%)');
    elseif i==6
        title('hours worked (%)');
    elseif i==7
        title('investment (%)');
    else
        title('interest rate (%)');
    end
    hold off
end
perc025=prctile(postdraws_impd,vquant(1)*100)';perc025=reshape(perc025,100,8);
perc975=prctile(postdraws_impd,vquant(2)*100)';perc975=reshape(perc975,100,8);
mean_imp=mean(postdraws_impd)';mean_imp=reshape(mean_imp,100,8);
%Graphs
figure;
for i=1:8
    hold on
    subplot(4,2,i)
    H1=plot([mean_imp(:,i) perc975(:,i) perc025(:,i)]);
    xlabel('periods');
    set(H1(1),'Color','k','LineStyle','-');
    set(H1(2),'Color','r','LineStyle','--');
    set(H1(3),'Color','r','LineStyle','--');
    if i==1
        title('capital (%)');
    elseif i==2
        title('productivity shock (%)');
    elseif i==3
        title('preferences shock (%)');
    elseif i==4
        title('consumption (%)');
    elseif i==5
        title('output (%)');
    elseif i==6
        title('hours worked (%)');
    elseif i==7
        title('investment (%)');
    else
        title('interest rate (%)');
    end
    hold off
end
