% Estimation of the linear regression model with Gibbs-Sampling
close all;
clear all;
clc;
%
% 1) Data generation
% Let the model be: y_{t} = x_{t} * BETA + e_{t}, donde e_{t} ~ N(0,SIG2)
%
BETA    = [2;0.9];   % population parameter (unknown for a frequentist)
SIG2    = 0.006;     % population parameter (unknown for a frequentist)
k       = 2;         % nunmebr of regressors
T       = 500;       % sample length 
e1      = randn(T+1000,1)*sqrt(SIG2); e1=e1(1001:end);  % realizations for the perturbation, from N(0,SIG2)
y       = zeros(T,1);    % we generate space to fill a realization for the dependent variable
x       = randn(T,k-1);  % We assume the explanatory variable to be iid normal N(0,1) 
X       = [ones(T,1) x];
% We compute the dependent variable
for t=1:T
    y(t) = X(t,:)*BETA + e1(t);
end
%
% MCO Estimation (to be used as a benchmark)
%
bols    = inv(X'*X)*(X'*y);
sigols  = ((y-X*bols)'*(y-X*bols))/(T-k);
%
% BAYESIAN ESTIMATION
% STEP 1: Establishing the distributions for beta and sigma
% We assume:  beta0 ~ N(b0,Sigma0)
%           1/sig20 ~ G(v0/2,delta0/2) where G is the Gamma distribution
%
beta0 = [1; 0.50];         % mean for the prior distribution for  beta
Sigma0 = [10 0;0 10];      % variance of the prior distribution for  beta
v0 = 0;                    % prior shape parameter for sigma: higher values tend to lower the posterior
delta0 = 0;                % prior scale parameter for sigma: zero implies a "flat" prior 
                           % i.e. no prior information
%
% Parameters for Gibbs sampler
nburn   = 8000;         % burn-in period: realizations to be ignored
nkeep   = 16000;        % number of realizations used for inference 
n       = nkeep+nburn;  % total number of iterations in Gibbs sampling
%
sig2    = 1;            % Initial value of sigma for Gibbs sampler
%
Beta    = zeros(nkeep,k); % Space to store realizations of the posterior for beta 
Sig2    = zeros(nkeep,1); % Space to store realizations of the posterior for sigma

for iter=1:n
    (fix(iter*100/n))
    % We generate a realization for beta, condicional on sig2
    Sigma1  = inv( inv(Sigma0)+1/sig2*(X'*X) );
    beta1  = Sigma1*(inv(Sigma0)*beta0+1/sig2*(X'*y));
%  
    beta = beta1 + chol(Sigma1)'*randn(k,1);
%
    % e generate a realization for sig2 conditional on beta
    v1      = v0+T;
    delta1  = delta0+(y-X*beta)'*(y-X*beta);
    shape    = v1/2;
    scale    = 1/(delta1/2);  
    sig2inv = gamrnd(shape,scale,1,1);
    sig2    = 1/sig2inv;
    %   
    if iter>nburn
        Beta(iter-nburn,:)  = beta';
        Sig2(iter-nburn,:)  = sig2;
    end
end

figure;
dim1 = 3;
dim2 = 2;
nbins = 500;

subplot(dim1,dim2,1)
histogram(Beta(:,1),nbins,'FaceColor','k','EdgeColor','k')
title('\fontsize{16} Intercept (\beta_{0})')
v=axis;
axis([0.9 2.1 v(3) v(4)])
ax = gca; 
ax.FontSize = 14;
line(BETA(1)*ones(1,2),[v(3) v(4)],'Linewidth',1,'Color','r')
line(beta0(1)*ones(1,2),[v(3) v(4)],'Linewidth',1,'Color','m')
line(bols(1)*ones(1,2),[v(3) v(4)],'Linewidth',1,'Color','b')
legend({'estimated','true','prior','b_{OLS}'},'FontSize',14)

subplot(dim1,dim2,2)
histogram(Beta(:,1),nbins,'FaceColor','k','EdgeColor','k')
title('\fontsize{16} Intercept (\beta_{0})')
v=axis;
ax = gca; 
ax.FontSize = 14;
line(BETA(1)*ones(1,2),[v(3) v(4)],'Linewidth',1,'Color','r')
line(mean(Beta(:,1))*ones(1,2),[v(3) v(4)],'Linewidth',2,'Color','g')
line(bols(1)*ones(1,2),[v(3) v(4)],'Linewidth',1,'Color','b')
legend({'estimated','true','posterior mean','b_{OLS}'},'FontSize',14)

subplot(dim1,dim2,3)
histogram(Beta(:,2),nbins,'FaceColor','k','EdgeColor','k')
title('\fontsize{16} Slope (\beta_{1})')
v=axis;
axis([0.4 1 v(3) v(4)])
ax = gca; 
ax.FontSize = 14;
line(BETA(2)*ones(1,2),[v(3) v(4)],'Linewidth',1,'Color','r')
line(beta0(2)*ones(1,2),[v(3) v(4)],'Linewidth',1,'Color','m')
line(bols(2)*ones(1,2),[v(3) v(4)],'Linewidth',1,'Color','b')
legend({'estimated','true','prior','b_{OLS}'},'FontSize',14)

subplot(dim1,dim2,4)
histogram(Beta(:,2),nbins,'FaceColor','k','EdgeColor','k')
title('\fontsize{16} Slope (\beta_{1})')
v=axis;
ax = gca; 
ax.FontSize = 14;
line(BETA(2)*ones(1,2),[v(3) v(4)],'Linewidth',1,'Color','r')
line(mean(Beta(:,2))*ones(1,2),[v(3) v(4)],'Linewidth',2,'Color','g')
line(bols(2)*ones(1,2),[v(3) v(4)],'Linewidth',1,'Color','b')
legend({'estimated','true','posterior mean','b_{OLS}'},'FontSize',14)

subplot(dim1,dim2,5)
histogram(Sig2,nbins,'FaceColor','k','EdgeColor','k')
title('\fontsize{16} Error Variance (\sigma^{2})')
v = axis;
ax = gca; 
ax.FontSize = 14;
line(SIG2*ones(1,2),[v(3) v(4)],'Linewidth',1,'Color','r')
line(mean(Sig2)*ones(1,2),[v(3) v(4)],'Linewidth',2,'Color','g')
line(sigols*ones(1,2),[v(3) v(4)],'Linewidth',1,'Color','b')
legend({'estimated','true','posterior mean','sig^{2}_{OLS}'},'FontSize',14)

